@@ -39,6 +39,7 @@ UNATTENDED_BOOT_STATUS_TYPE = make_enum_type(pynitrokey.nethsm.UnattendedBootSta
 TYPE_TYPE = make_enum_type(pynitrokey.nethsm.KeyType)
 TYPE_TLS_KEY_TYPE = make_enum_type(pynitrokey.nethsm.TlsKeyType)
 MECHANISM_TYPE = make_enum_type(pynitrokey.nethsm.KeyMechanism)
+ENCRYPT_MODE_TYPE = make_enum_type(pynitrokey.nethsm.EncryptMode)
 DECRYPT_MODE_TYPE = make_enum_type(pynitrokey.nethsm.DecryptMode)
 SIGN_MODE_TYPE = make_enum_type(pynitrokey.nethsm.SignMode)
 
@@ -220,7 +221,7 @@ def list_users(ctx, details):
             headers += ["Real name", "Role"]
             data = []
             for user_id in user_ids:
-                user = nethsm.get_user(user_id=user_id.value)
+                user = nethsm.get_user(user_id=user_id)
                 data.append([user_id, user.real_name, user.role.value])
         else:
             data = [[user_id] for user_id in user_ids]
@@ -318,9 +319,9 @@ def list_operator_tags(ctx, user_id):
     This command requires authentication as a user with the Administrator role."""
     with connect(ctx) as nethsm:
         tags = nethsm.list_operator_tags(user_id=user_id)
-        if tags.value:
+        if tags:
             print(f"Tags for user {user_id}:")
-            for tag in tags.value:
+            for tag in tags:
                 print(f"- {tag}")
         else:
             print(f"No tags set for user {user_id}.")
@@ -431,14 +432,20 @@ def metrics(ctx):
     default=True,
     help="Also query the key data",
 )
+@click.option(
+    "-f",
+    "--filter",
+    type=str,
+    help="Filter keys by tags for respective user",
+)
 @click.pass_context
-def list_keys(ctx, details):
+def list_keys(ctx, details, filter):
     """List all keys on the NetHSM.
 
     This command requires authentication as a user with the Administrator or
     Operator role."""
     with connect(ctx) as nethsm:
-        key_ids = nethsm.list_keys()
+        key_ids = nethsm.list_keys(filter)
 
         print(f"Keys on NetHSM {nethsm.host}:")
         print()
@@ -448,7 +455,7 @@ def list_keys(ctx, details):
             headers += ["Type", "Mechanisms", "Operations", "Tags"]
             data = []
             for key_id in key_ids:
-                key = nethsm.get_key(key_id=key_id.value)
+                key = nethsm.get_key(key_id=key_id)
                 data.append(
                     [
                         key_id,
@@ -576,6 +583,12 @@ def prompt_mechanisms(type):
     multiple=True,
     help="The mechanisms for the new key",
 )
+@click.option(
+    "--tags",
+    type=str,
+    multiple=True,
+    help="The tags for the new key",
+)
 @click.option(
     "-p",
     "--prime-p",
@@ -602,7 +615,9 @@ def prompt_mechanisms(type):
     help="The ID of the new key",
 )
 @click.pass_context
-def add_key(ctx, type, mechanisms, prime_p, prime_q, public_exponent, data, key_id):
+def add_key(
+    ctx, type, mechanisms, tags, prime_p, prime_q, public_exponent, data, key_id
+):
     """Add a key pair on the NetHSM.
 
     If the key ID is not set, it is generated by the NetHSM.
@@ -637,6 +652,7 @@ def add_key(ctx, type, mechanisms, prime_p, prime_q, public_exponent, data, key_
             key_id=key_id,
             type=type,
             mechanisms=mechanisms,
+            tags=tags,
             prime_p=prime_p,
             prime_q=prime_q,
             public_exponent=public_exponent,
@@ -711,14 +727,14 @@ def get_config(ctx, **kwargs):
         if show_all or kwargs["logging"]:
             data = nethsm.get_config_logging()
             print("  Logging:")
-            print("    IP address:   ", data.ip_address)
+            print("    IP address:   ", data.ipAddress)
             print("    Port:         ", data.port)
-            print("    Log level:    ", data.log_level)
+            print("    Log level:    ", data.logLevel)
 
         if show_all or kwargs["network"]:
             data = nethsm.get_config_network()
             print("  Network:")
-            print("    IP address:   ", data.ip_address)
+            print("    IP address:   ", data.ipAddress)
             print("    Netmask:      ", data.netmask)
             print("    Gateway:      ", data.gateway)
 
@@ -1266,6 +1282,45 @@ def factory_reset(ctx, force):
             print(f"Factory reset on NetHSM {nethsm.host} cancelled")
 
 
+@nethsm.command()
+@click.option(
+    "-k",
+    "--key-id",
+    prompt=True,
+    help="The ID of the key to encrypt the data with",
+)
+@click.option(
+    "-d",
+    "--data",
+    prompt=True,
+    help="The data in Base64 encoding",
+)
+@click.option(
+    "-m",
+    "--mode",
+    type=ENCRYPT_MODE_TYPE,
+    prompt=True,
+    help="The encrypt mode",
+)
+@click.option(
+    "-iv",
+    "--initialization-vector",
+    "iv",
+    type=str,
+    prompt=True,
+    help="The initialization vector",
+)
+@click.pass_context
+def encrypt(ctx, key_id, data, mode, iv):
+    """Encrypt data with an asymmetric secret key on the NetHSM and print the encrypted message.
+
+    This command requires authentication as a user with the Operator role."""
+    with connect(ctx) as nethsm:
+        encrypted = nethsm.encrypt(key_id, data, mode, iv)
+        print(f"Encrypted: {encrypted[0]}")
+        print(f"Initialization vector: {encrypted[1]}")
+
+
 @nethsm.command()
 @click.option(
     "-k",
