@@ -19,7 +19,12 @@ import requests
 
 import pynitrokey
 from pynitrokey.confconsts import LOG_FN
-from pynitrokey.helpers import AskUser, local_critical, local_print
+from pynitrokey.helpers import (
+    AskUser,
+    check_pynitrokey_version,
+    local_critical,
+    local_print,
+)
 
 logger = logging.getLogger()
 
@@ -41,6 +46,8 @@ def update(serial, yes, force):
     # print('Please use {} to run the firmware update'.format(update_url))
     # return
 
+    check_pynitrokey_version()
+
     IS_LINUX = platform.system() == "Linux"
 
     logger.debug(f"Start session {datetime.now()}")
@@ -166,7 +173,7 @@ def update(serial, yes, force):
             local_critical("exiting due to user input...", support_hint=False)
 
     # Ensure we are in bootloader mode
-    if client.is_solo_bootloader():
+    if client.is_bootloader():
         local_print("Key already in bootloader mode, continuing...")
     else:
         try:
