@@ -17,12 +17,13 @@ import sys
 import tempfile
 import time
 from getpass import getpass
-from typing import Any, Optional
+from typing import Any, Callable, List, Optional, Tuple, Union
 
 from fido2.client import Fido2Client, UserInteraction
 from fido2.ctap import CtapError
 from fido2.ctap1 import Ctap1
 from fido2.ctap2 import CredentialManagement, Ctap2
+from fido2.ctap2.credman import CredentialManagement
 from fido2.ctap2.pin import ClientPin
 from fido2.hid import CTAPHID, CtapHidDevice, open_device
 from intelhex import IntelHex
@@ -52,45 +53,45 @@ class CliOrProvidedInteraction(UserInteraction):
 
 
 class NKFido2Client:
-    def __init__(
-        self,
-    ):
+    def __init__(self) -> None:
         self.origin = "https://example.org"
         self.host = "example.org"
         self.user_id = b"they"
         self.exchange = self.exchange_hid
         self.do_reboot = True
 
-    def use_u2f(
-        self,
-    ):
+    def use_u2f(self) -> None:
         self.exchange = self.exchange_u2f
 
-    def use_hid(
-        self,
-    ):
+    def use_hid(self) -> None:
         self.exchange = self.exchange_hid
 
-    def set_reboot(self, val):
+    def set_reboot(self, val: bool) -> None:
         """option to reboot after programming"""
         self.do_reboot = val
 
-    def reboot(
-        self,
-    ):
+    def reboot(self) -> None:
         """option to reboot after programming"""
         try:
             self.exchange(SoloBootloader.reboot)
         except OSError:
             pass
 
-    def find_device(self, dev=None, solo_serial: Optional[str] = None, pin=None):
+    def find_device(
+        self,
+        dev: Optional[CtapHidDevice] = None,
+        solo_serial: Optional[str] = None,
+        pin: Optional[str] = None,
+    ) -> CtapHidDevice:
+
         devices = []
+        found_dev: Optional[CtapHidDevice] = None
+
         if dev is None:
             if solo_serial is not None:
                 if solo_serial.startswith("device="):
                     solo_serial = solo_serial.split("=")[1]
-                    dev = open_device(solo_serial)
+                    found_dev = open_device(solo_serial)
                 else:
                     devices = list(CtapHidDevice.list_devices())
                     devices = [
@@ -101,53 +102,56 @@ class NKFido2Client:
             if len(devices) > 1:
                 raise pynitrokey.exceptions.NonUniqueDeviceError
             if len(devices) > 0:
-                dev = devices[0]
-        if dev is None:
+                found_dev = devices[0]
+        if dev is None and found_dev is None:
             raise RuntimeError("No FIDO device found")
-        self.dev = dev
 
-        self.ctap1 = Ctap1(dev)
+        self.dev = found_dev if found_dev else dev
+        assert isinstance(self.dev, CtapHidDevice)
+        self.ctap1 = Ctap1(self.dev)
 
         try:
-            self.ctap2: Optional[Ctap2] = Ctap2(dev)
+            self.ctap2: Optional[Ctap2] = Ctap2(self.dev)
         except CtapError as e:
             self.ctap2 = None
 
         try:
             self.client: Optional[Fido2Client] = Fido2Client(
-                dev, self.origin, user_interaction=CliOrProvidedInteraction(pin)
+                self.dev, self.origin, user_interaction=CliOrProvidedInteraction(pin)
             )
         except CtapError:
             print("Not using FIDO2 interface.")
             self.client = None
 
         if self.exchange == self.exchange_hid:
-            self.send_data_hid(CTAPHID.INIT, "\x11\x11\x11\x11\x11\x11\x11\x11")
+            self.send_data_hid(CTAPHID.INIT, b"\x11\x11\x11\x11\x11\x11\x11\x11")
 
         return self.dev
 
     @staticmethod
-    def format_request(cmd, addr=0, data=b"A" * 16):
+    def format_request(cmd: int, addr: int = 0, data: bytes = b"A" * 16) -> bytes:
         # not sure why this is here?
         # arr = b"\x00" * 9
-        addr = struct.pack("<L", addr)
-        cmd = struct.pack("B", cmd)
+        packed_addr = struct.pack("<L", addr)
+        packed_cmd = struct.pack("B", cmd)
         length = struct.pack(">H", len(data))
 
-        return cmd + addr[:3] + SoloBootloader.TAG + length + data
+        return packed_cmd + packed_addr[:3] + SoloBootloader.TAG + length + data
 
-    def send_only_hid(self, cmd, data):
+    def send_only_hid(self, cmd: int, data: bytes = b"A" * 16) -> None:
         if not isinstance(data, bytes):
             data = struct.pack("%dB" % len(data), *[ord(x) for x in data])
-        self.dev._dev.InternalSend(0x80 | cmd, bytearray(data))
+        assert isinstance(self.dev, CtapHidDevice)
+        self.dev.call(0x80 | cmd, bytearray(data))
 
-    def send_data_hid(self, cmd, data):
+    def send_data_hid(self, cmd: int, data: bytes = b"A" * 16) -> bytes:
         if not isinstance(data, bytes):
             data = struct.pack("%dB" % len(data), *[ord(x) for x in data])
         with helpers.Timeout(1.0) as event:
-            return self.dev.call(cmd, data, event)
+            assert isinstance(self.dev, CtapHidDevice)
+            return self.dev.call(cmd, data, event=event)
 
-    def exchange_hid(self, cmd, addr=0, data=b"A" * 16):
+    def exchange_hid(self, cmd: int, addr: int = 0, data: bytes = b"A" * 16) -> bytes:
         req = NKFido2Client.format_request(cmd, addr, data)
 
         data = self.send_data_hid(SoloBootloader.HIDCommandBoot, req)
@@ -158,7 +162,7 @@ class NKFido2Client:
 
         return data[1:]
 
-    def exchange_u2f(self, cmd, addr=0, data=b"A" * 16):
+    def exchange_u2f(self, cmd: int, addr: int = 0, data: bytes = b"A" * 16) -> bytes:
         appid = b"A" * 32
         chal = b"B" * 32
 
@@ -172,11 +176,12 @@ class NKFido2Client:
 
         return res.signature[1:]
 
-    def exchange_fido2(self, cmd, addr=0, data=b"A" * 16):
+    def exchange_fido2(self, cmd: int, addr: int = 0, data: bytes = b"A" * 16) -> bytes:
         chal = b"B" * 32
 
         req = NKFido2Client.format_request(cmd, addr, data)
 
+        assert isinstance(self.ctap2, Ctap2)
         assertion = self.ctap2.get_assertion(
             self.host, chal, [{"id": req, "type": "public-key"}]
         )
@@ -188,39 +193,35 @@ class NKFido2Client:
 
         return res.signature[1:]
 
-    def bootloader_version(
-        self,
-    ):
+    def bootloader_version(self) -> Tuple[int, int, int]:
         data = self.exchange(SoloBootloader.version)
         if len(data) > 2:
             return (data[0], data[1], data[2])
         return (0, 0, data[0])
 
-    def solo_version(
-        self,
-    ):
+    def solo_version(self) -> Union[bytes, Tuple[int, int, int]]:
         try:
             return self.send_data_hid(0x61, b"")
         except CtapError:
             data = self.exchange(SoloExtension.version)
             return (data[0], data[1], data[2])
 
-    def write_flash(self, addr, data):
+    def write_flash(self, addr: int, data: bytes) -> None:
         self.exchange(SoloBootloader.write, addr, data)
 
-    def boot_pubkey(self):
+    def boot_pubkey(self) -> bytes:
         return self.exchange(SoloBootloader.boot_pubkey)
 
-    def get_rng(self, num=0):
+    def get_rng(self, num: int = 0) -> bytes:
         ret = self.send_data_hid(SoloBootloader.HIDCommandRNG, struct.pack("B", num))
         return ret
 
-    def get_status(self, num=0):
+    def get_status(self, num: int = 0) -> bytes:
         ret = self.send_data_hid(SoloBootloader.HIDCommandStatus, struct.pack("B", num))
         # print(ret[:8])
         return ret
 
-    def verify_flash(self, sig):
+    def verify_flash(self, sig: bytes) -> None:
         """
         Tells device to check signature against application.  If it passes,
         the application will boot.
@@ -228,40 +229,42 @@ class NKFido2Client:
         """
         self.exchange(SoloBootloader.done, 0, sig)
 
-    def wink(
-        self,
-    ):
+    def wink(self) -> None:
         self.send_data_hid(CTAPHID.WINK, b"")
 
-    def reset(
-        self,
-    ):
+    def reset(self) -> None:
+        assert isinstance(self.ctap2, Ctap2)
         self.ctap2.reset()
 
     def make_credential(
         self,
-        host="nitrokeys.dev",
-        user_id="they",
-        serial=None,
-        output=True,
-        udp=False,
-        fingerprint_only=False,
-    ):
+        host: str = "nitrokeys.dev",
+        user_id: str = "they",
+        serial: Optional[str] = None,
+        pin: Optional[str] = None,
+        prompt: str = "Touch your authenticator to generate a credential...",
+        output: bool = True,
+        udp: bool = False,
+        fingerprint_only: bool = False,
+    ) -> str:
+
         """
         fingerprint_only bool Return sha256 digest of the certificate, in a hex string format. Useful for detecting
             device's model and firmware.
         """
 
-        user_id = user_id.encode()
+        _user_id = user_id.encode()
+        assert isinstance(self.client, Fido2Client)
         client = self.client
 
+        # @todo: rewrite with typing; use fido2.webauthn.PublicKeyCredentialCreationOptions
         rp = {"id": host, "name": "Example RP"}
-        client.host = host
-        client.origin = f"https://{client.host}"
-        client.user_id = user_id
-        user = {"id": user_id, "name": "A. User"}
+        client.host = host  # type: ignore
+        client.origin = f"https://{client.host}"  # type: ignore
+        client.user_id = _user_id  # type: ignore
+        user = {"id": _user_id, "name": "A. User"}
         challenge = secrets.token_bytes(32)
-
+        # @todo: ... pass PublicKeyCredentialCreationOptions here
         attestation_object = client.make_credential(
             {
                 "rp": rp,
@@ -272,7 +275,7 @@ class NKFido2Client:
                     {"type": "public-key", "alg": -7},
                 ],
                 "extensions": {"hmacCreateSecret": True},
-            },
+            },  # type: ignore
         ).attestation_object
 
         if fingerprint_only:
@@ -284,36 +287,39 @@ class NKFido2Client:
             return sha256(data[0]).digest().hex()
 
         credential = attestation_object.auth_data.credential_data
-        credential_id = credential.credential_id
+        # @todo: this generally will not work, fix this
+        credential_id = credential.credential_id  # type: ignore
         if output:
             print(credential_id.hex())
 
-        return credential_id
+        return credential_id.hex()
 
     def simple_secret(
         self,
-        credential_id,
-        secret_input,
-        host="nitrokeys.dev",
-        user_id="they",
-        serial=None,
-        pin=None,
-        prompt="Touch your authenticator to generate a response...",
-        output=True,
-        udp=False,
-    ):
-        user_id = user_id.encode()
+        credential_id: str,
+        secret_input: str,
+        host: str = "nitrokeys.dev",
+        user_id: str = "they",
+        serial: Optional[str] = None,
+        prompt: Optional[str] = "Touch your authenticator to generate a response...",
+        output: bool = True,
+        udp: bool = False,
+    ) -> bytes:
 
+        _user_id = user_id.encode()
+
+        assert isinstance(self.client, Fido2Client)
         client = self.client
 
+        # @todo: rewrite with typing; use fido2.webauthn.PublicKeyCredentialCreationOptions
         # rp = {"id": host, "name": "Example RP"}
-        client.host = host
-        client.origin = f"https://{client.host}"
-        client.user_id = user_id
+        client.host = host  # type: ignore
+        client.origin = f"https://{client.host}"  # type: ignore
+        client.user_id = _user_id  # type: ignore
         # user = {"id": user_id, "name": "A. User"}
-        credential_id = binascii.a2b_hex(credential_id)
+        _credential_id = binascii.a2b_hex(credential_id)
 
-        allow_list = [{"type": "public-key", "id": credential_id}]
+        allow_list = [{"type": "public-key", "id": _credential_id}]
 
         challenge = secrets.token_bytes(32)
 
@@ -330,18 +336,24 @@ class NKFido2Client:
                 "challenge": challenge,
                 "allowCredentials": allow_list,
                 "extensions": {"hmacGetSecret": {"salt1": salt}},
-            },
-            pin=pin,
+            },  # type: ignore
         ).get_response(0)
 
-        output = assertion.extension_results["hmacGetSecret"]["output1"]
+        # @todo: rewrite with typing
+        output = assertion.extension_results["hmacGetSecret"]["output1"]  # type: ignore
+        assert isinstance(output, bytes)
         if output:
             print(output.hex())
 
         return output
 
-    def cred_mgmt(self, serial, pin):
+    def has_pin(self) -> bool:
+        assert self.client is not None
+        return self.client.info.options["clientPin"]
+
+    def cred_mgmt(self, serial: str, pin: str) -> CredentialManagement:
         device = nkfido2.find(serial)
+        assert isinstance(device.ctap2, Ctap2)
         client_pin = ClientPin(device.ctap2)
 
         try:
@@ -362,21 +374,19 @@ class NKFido2Client:
 
         return CredentialManagement(device.ctap2, client_pin.protocol, client_token)
 
-    def enter_solo_bootloader(
-        self,
-    ):
+    def enter_bootloader(self) -> None:
         """
         If Nitrokey is configured as Nitrokey hacker or something similar,
         this command will tell the token to boot directly to the bootloader
         so it can be reprogrammed
         """
         if self.exchange != self.exchange_hid:
-            self.send_data_hid(CTAPHID.INIT, "\x11\x11\x11\x11\x11\x11\x11\x11")
-        self.send_data_hid(SoloBootloader.HIDCommandEnterBoot, "")
+            self.send_data_hid(CTAPHID.INIT, b"\x11\x11\x11\x11\x11\x11\x11\x11")
+        self.send_data_hid(SoloBootloader.HIDCommandEnterBoot, b"")
 
-    def enter_bootloader_or_die(self):
+    def enter_bootloader_or_die(self) -> None:
         try:
-            self.enter_solo_bootloader()
+            self.enter_bootloader()
         # except OSError:
         #     pass
         except CtapError as e:
@@ -388,9 +398,7 @@ class NKFido2Client:
             else:
                 raise (e)
 
-    def is_solo_bootloader(
-        self,
-    ):
+    def is_bootloader(self) -> bool:
         try:
             self.bootloader_version()
             return True
@@ -405,60 +413,42 @@ class NKFido2Client:
             pass
         return False
 
-    def enter_st_dfu(
-        self,
-    ):
+    def enter_st_dfu(self) -> None:
         """
         If Nitrokey is configured as Nitrokey hacker or something similar,
         this command will tell the token to boot directly to the st DFU
         so it can be reprogrammed.  Warning, you could brick your device.
         """
-        soloboot = self.is_solo_bootloader()
+        boot = self.is_bootloader()
 
-        if soloboot or self.exchange == self.exchange_u2f:
+        if boot or self.exchange == self.exchange_u2f:
             req = NKFido2Client.format_request(SoloBootloader.st_dfu)
             self.send_only_hid(SoloBootloader.HIDCommandBoot, req)
         else:
-            self.send_only_hid(SoloBootloader.HIDCommandEnterSTBoot, "")
+            self.send_only_hid(SoloBootloader.HIDCommandEnterSTBoot, b"")
 
-    def disable_solo_bootloader(
-        self,
-    ):
-        """
-        Disables the Nitrokey bootloader.  Only do this if you want to void the possibility
-        of any updates.
-        If you've started from a Nitrokey hacker, make you you've programmed a final/production build!
-        """
-        ret = self.exchange(
-            SoloBootloader.disable, 0, b"\xcd\xde\xba\xaa"
-        )  # magic number
-        if ret[0] != CtapError.ERR.SUCCESS:
-            print("Failed to disable bootloader")
-            return False
-        time.sleep(0.1)
-        self.exchange(SoloBootloader.do_reboot)
-        return True
-
-    def program_file(self, name):
-        def parseField(f):
+    def program_file(self, name: str) -> bytes:
+        def parseField(f: str) -> bytes:
             return base64.b64decode(helpers.from_websafe(f).encode())
 
-        def isCorrectVersion(current, target):
+        def isCorrectVersion(current: Tuple[int, int, int], target: str) -> bool:
             """current is tuple (x,y,z).  target is string '>=x.y.z'.
             Return True if current satisfies the target expression.
             """
             if "=" in target:
-                target = target.split("=")
-                assert target[0] in [">", "<"]
-                target_num = [int(x) for x in target[1].split(".")]
-                assert len(target_num) == 3
-                comp = target[0] + "="
+                target_toks = target.split("=")
+                assert target_toks[0] in [">", "<"]
+                target_num_toks = [int(x) for x in target_toks[1].split(".")]
+                assert len(target_num_toks) == 3
+                comp = target_toks[0] + "="
             else:
                 assert target[0] in [">", "<"]
-                target_num = [int(x) for x in target[1:].split(".")]
+                target_num_toks = [int(x) for x in target[1:].split(".")]
                 comp = target[0]
             target_num = (
-                (target_num[0] << 16) | (target_num[1] << 8) | (target_num[2] << 0)
+                (target_num_toks[0] << 16)
+                | (target_num_toks[1] << 8)
+                | (target_num_toks[2] << 0)
             )
             current_num = (current[0] << 16) | (current[1] << 8) | (current[2] << 0)
             return eval(str(current_num) + comp + str(target_num))
@@ -517,8 +507,8 @@ class NKFido2Client:
         print("erasing firmware...")
         for i in range(seg[0], seg[1], chunk):
             s = i
-            e = min(i + chunk, seg[1])  # type: ignore
-            data = ih.tobinarray(start=i, size=e - s)
+            ext = min(i + chunk, seg[1])
+            data = ih.tobinarray(start=i, size=ext - s)
             self.write_flash(i, data)
             total += chunk
             progress = total / float(size) * 100
@@ -557,7 +547,7 @@ class NKFido2Client:
 
         return sig
 
-    def check_only(self, name):
+    def check_only(self, name: str) -> None:
         # FIXME refactor
         # copy from program_file
         if name.lower().endswith(".json"):
