@@ -15,14 +15,18 @@ import platform
 import sys
 import time
 from getpass import getpass
+from importlib.metadata import version
+from itertools import chain
 from numbers import Number
 from threading import Event, Timer
-from typing import List, Optional
+from typing import Any, Callable, Dict, List, NoReturn, Optional, Tuple, TypeVar, Union
 
 import click
+import semver
 from tqdm import tqdm
 
 from pynitrokey.confconsts import (
+    CLI_LOG_BLACKLIST,
     LOG_FN,
     SUPPORT_EMAIL,
     SUPPORT_URL,
@@ -30,18 +34,58 @@ from pynitrokey.confconsts import (
     VERBOSE,
     Verbosity,
 )
+from pynitrokey.updates import Repository
 
 STDOUT_PRINT = True
 
 
-def to_websafe(data):
+def normalize_parameters(s: str) -> list[str]:
+    """Helper function to normalize different writing of parameters
+
+    `s`: `str` parameter to normalize
+
+    Returns:
+        `list of str`: normalized form of `parameter`
+    """
+    if s.startswith("--"):
+        return s.split("=", maxsplit=1)
+    else:
+        return [s]
+
+
+def filter_sensitive_parameters(parameters: list[str]) -> list[str]:
+    """Helper function to remove sensitive parameters
+
+    `parameters`: `list of str`
+
+    Returns:
+        `list of str`: `parameters` without sensitive values listed
+                       in `pynitrokey.confconsts.CLI_LOG_BLACKLIST`
+    """
+    parameters = list(
+        chain.from_iterable(
+            [normalize_parameters(parameter) for parameter in parameters]
+        )
+    )
+
+    redact_count = 0
+    for i in range(len(parameters)):
+        if redact_count > 0:
+            parameters[i] = "[redacted]"
+            redact_count -= 1
+        elif parameters[i] in CLI_LOG_BLACKLIST:
+            redact_count = CLI_LOG_BLACKLIST[parameters[i]]
+    return parameters
+
+
+def to_websafe(data: str) -> str:
     data = data.replace("+", "-")
     data = data.replace("/", "_")
     data = data.replace("=", "")
     return data
 
 
-def from_websafe(data):
+def from_websafe(data: str) -> str:
     data = data.replace("-", "+")
     data = data.replace("_", "/")
     return data + "=="[: (3 * len(data)) % 4]
@@ -53,7 +97,7 @@ class ProgressBar:
     is not available before the first iteration.
     """
 
-    def __init__(self, **kwargs) -> None:
+    def __init__(self, **kwargs: Any) -> None:
         self.bar: Optional[tqdm] = None
         self.kwargs = kwargs
         self.sum = 0
@@ -61,7 +105,7 @@ class ProgressBar:
     def __enter__(self) -> "ProgressBar":
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
+    def __exit__(self, exc_type: None, exc_val: None, exc_tb: None) -> None:
         self.close()
 
     def update(self, n: int, total: int) -> None:
@@ -99,20 +143,20 @@ class Timeout(object):
     :ivar timer: The Timer associated with the Timeout, if any.
     """
 
-    def __init__(self, time_or_event):
-        if isinstance(time_or_event, Number):
+    def __init__(self, time_or_event: Union[float, Event]) -> None:
+        if isinstance(time_or_event, float):
             self.event = Event()
-            self.timer = Timer(time_or_event, self.event.set)
+            self.timer: Optional[Timer] = Timer(float(time_or_event), self.event.set)
         else:
             self.event = time_or_event
             self.timer = None
 
-    def __enter__(self):
+    def __enter__(self) -> Event:
         if self.timer:
             self.timer.start()
         return self.event
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type: None, exc_val: None, exc_tb: None) -> None:
         if self.timer:
             self.timer.join()
             self.timer.cancel()
@@ -156,13 +200,9 @@ class Retries:
 # @todo: introduce granularization: dbg, info, err (warn?)
 #        + machine-readable
 #        + logfile-only (partly solved)
-def local_print(*messages, **kwargs):
-    """
-    application-wide logging function
-    `messages`:   `str`         -> log single string
-                  `Exception`   -> log exception
-                  `list of ...` -> list of either `str` or `Exception` handle serialized
-    """
+def local_print(*messages: Any, **kwargs: Any) -> None:
+    """Application-wide logging function"""
+
     passed_exc = None
     logger = logging.getLogger()
 
@@ -192,10 +232,14 @@ def local_print(*messages, **kwargs):
         raise passed_exc
 
 
-def local_critical(*messages, support_hint=True, ret_code=1, **kwargs):
+def local_critical(
+    *messages: Any, support_hint: bool = True, ret_code: int = 1, **kwargs: Any
+) -> None:
+
     global STDOUT_PRINT
-    messages = ["Critical error:"] + list(messages)
+    messages = ("Critical error:",) + tuple(messages)
     local_print(*messages, **kwargs)
+
     if support_hint:
 
         # list all connected devices to logfile
@@ -206,7 +250,8 @@ def local_critical(*messages, support_hint=True, ret_code=1, **kwargs):
         try:
             from pynitrokey.cli import nitropy
 
-            nitropy.commands["list"].callback()
+            nitropy.commands["list"].callback()  # type: ignore
+
         except Exception:
             local_print("Unable to list devices. See log for the details.")
             logger = logging.getLogger()
@@ -251,12 +296,12 @@ class AskUser:
         options: List[str] = [],
         strict: bool = False,
         repeat: int = 3,
-        adapt_question=True,
-        hide_input=False,
+        adapt_question: bool = True,
+        hide_input: bool = False,
         envvar: Optional[str] = None,
-    ):
+    ) -> None:
 
-        self.data = None
+        self.data: Optional[str] = None
 
         self.question = question
         self.adapt_question = adapt_question
@@ -281,32 +326,36 @@ class AskUser:
         self.envvar = envvar
 
     @classmethod
-    def yes_no(cls, what: str, strict: bool = False):
+    def yes_no(cls, what: str, strict: bool = False) -> bool:
         opts = ["yes", "no"]
         return cls(what, options=opts, strict=strict).ask() == opts[0]
 
     @classmethod
-    def strict_yes_no(cls, what: str):
+    def strict_yes_no(cls, what: str) -> bool:
         return cls.yes_no(what, strict=True)
 
     @classmethod
-    def plain(cls, what):
+    def plain(cls, what: str) -> str:
         return cls(what).ask()
 
     @classmethod
-    def hidden(cls, what):
+    def hidden(cls, what: str) -> str:
         return cls(what, hide_input=True).ask()
 
-    def get_input(self, pre_str=None, hide_input=None):
+    def get_input(
+        self,
+        pre_str: Optional[str] = None,
+        hide_input: Optional[Union[str, bool]] = None,
+    ) -> str:
         pre_input_string = pre_str or self.final_question
         hide_input = hide_input if hide_input is not None else self.hide_input
         if hide_input:
             return getpass(pre_input_string)
         else:
             print(pre_input_string, end="", file=sys.stderr)
-            return input(pre_input_string).strip()
+            return input().strip()
 
-    def ask(self):
+    def ask(self) -> str:
         answer = None
         if self.envvar is not None:
             fromvar = os.environ.get(self.envvar)
@@ -344,7 +393,7 @@ class AskUser:
             local_critical("max tries exceeded - exiting...")
 
         assert self.data is None, "expecting `self.data` to be None at this point!"
-        return self.data
+        return self.data or ""
 
 
 confirm = functools.partial(click.confirm, err=True)
@@ -358,3 +407,22 @@ def require_windows_admin() -> None:
                 "Warning: It is recommended to execute nitropy with admin privileges "
                 "to be able to access Nitrokey 3 and Nitrokey FIDO 2 devices."
             )
+
+
+def check_pynitrokey_version() -> None:
+    """Checks wether the used pynitrokey version is the latest available version and warns the user if the used version is outdated"""
+
+    latest_release = Repository("Nitrokey", "pynitrokey").get_latest_release()
+    latest_version = semver.Version.parse(latest_release.tag[1:])
+
+    current_version = semver.Version.parse(version("pynitrokey"))
+
+    if current_version < latest_version:
+        local_print(
+            f"You are using an outdated version ({current_version}) of pynitrokey."
+        )
+        local_print(f"Latest pynitrokey version is {latest_version}")
+        local_print("Updating with an outdated version is discouraged.")
+
+        if not confirm("Do you still want to continue?", default=False):
+            raise click.Abort()
