@@ -100,16 +100,11 @@ class Nitrokey3BootloaderLpc55(Nitrokey3Bootloader):
         callback: Optional[ProgressCallback] = None,
         check_errors: bool = False,
     ) -> None:
-        # This is a workaround for a wrong type annotation for progress_callback (not Optional)
-        # in spsdk (fixed in 1.9.0).
-        if callback:
-            success = self.device.receive_sb_file(
-                image,
-                progress_callback=callback,
-                check_errors=check_errors,
-            )
-        else:
-            success = self.device.receive_sb_file(image, check_errors=check_errors)
+        success = self.device.receive_sb_file(
+            image,
+            progress_callback=callback,
+            check_errors=check_errors,
+        )
         logger.debug(f"Firmware update finished with status {self.status}")
         if success:
             self.reboot()
@@ -128,8 +123,6 @@ class Nitrokey3BootloaderLpc55(Nitrokey3Bootloader):
         )
         devices = []
         for device in RawHid.enumerate(device_filter):
-            # TODO: remove assert if https://github.com/NXPmicro/spsdk/issues/32 is fixed
-            assert isinstance(device, RawHid)
             try:
                 devices.append(Nitrokey3BootloaderLpc55(device))
             except ValueError:
@@ -150,8 +143,6 @@ class Nitrokey3BootloaderLpc55(Nitrokey3Bootloader):
             return None
 
         try:
-            # TODO: remove assert if https://github.com/NXPmicro/spsdk/issues/32 is fixed
-            assert isinstance(devices[0], RawHid)
             return Nitrokey3BootloaderLpc55(devices[0])
         except ValueError:
             logger.warn(
